<#compress>
<#--
   FreeMarker template to produce a HTML representation of a
   Statestep model.
   Copyright (c) 2006, Michael Breen.
   Open Source. Licensed under the Apache License, Version 2.0.
   See accompanying files for more details.
-->
<#-- Variables without any values must be ignored. -->
<#if doc.model.variables?size != 0>
   <#assign usable_variables =
      doc.model.variables["variable[./value]"]
   />
<#else>
   <#assign usable_variables = []/>
</#if>
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<meta name="generator" content="http://statestep.com/">
<TITLE><@title/></TITLE>
<STYLE type="text/css">
BODY { font-family: sans-serif; }
H1, H3, H4 { font-weight: normal }
H1 { text-align: center; font-size: 300% }
H2, H3 { font-size: 200%; margin-top: 1.5em }
H3 { font-style: italic; border-top: thin solid }
H4
{
   margin-top: 1.5em;
   text-decoration: underline;
   font-size: 150%
}
TABLE
{
   empty-cells: show;
   border-collapse: collapse;
   border-spacing: 0
}
TABLE, TH, TD
{
   background: white;
   border: 1px solid black
}
TD { vertical-align: top }
TH, TD.constraining { background: #DDD }
DIV.contents
{
   padding: 1em;
   border: 1px solid;
   margin: 1em 0;
   background: #DDD
}
UL.contents { list-style-type: none }
DT { font-weight: bold; }
.constraint-list DT, .rule-list DT, .variable-list DT
{
   margin-top: 1.5em; font-size: 120%
}
DD { margin-left: 0; margin-bottom: 1em }
.constraint-list DD, .rule-list DD { padding: 0.5em }
.variable-list DT { margin-bottom: 0.5em; }
.variable-list DD { margin-bottom: 0.5em; }
.value-list DT
{
   margin-top: 0.5em;
   margin-bottom: 0em;
   font-size: 100%
}
DL DL { margin-left: 3em; }
DL DL DD { margin-left: 2em; }
.value-list DD { margin-bottom: 0em }
DD.constraint-global { background: #DD9 }
DD.constraint-event { background: #CF9 }
DD.rule { background: #DDC }
PRE { margin: 0.5em 0 }
</STYLE>
</HEAD>

<BODY>
<H1><@title/></H1>
<ADDRESS>
Model created using <A href="http://statestep.com">Statestep</A>.
</ADDRESS>
<#if doc.model.description?size != 0>
   <P>
   ${doc.model.description?html}
   </P>
</#if>

<DIV class="contents">
<P><STRONG>Contents</STRONG></P>
<UL class="contents">
<LI><A href="#vars">Variables</A></LI>
<LI>
<A href="#globals">Global Constraints</A>
<UL class="contents">
<LI><A href="#ctable">Constraint Table</A></LI>
<LI><A href="#textglobals">Other Global Constraints</A></LI>
</UL>
</LI>
<LI>
<A href="#events">Events: Constraints and Rules</A>
<UL class="contents">
<#list doc.model.events.event as event>
<LI><A href="#event-${event.@name}">${event.@name}</A></LI>
</#list>
</UL>
</LI>
</UL>
</DIV>

<H2><A name="vars">Variables</A></H2>
<DL class="variable-list">
<#list usable_variables as var>
   <DT class="variable">${var.@name}</DT>
   <DD>
   <#if var.description?size != 0>
      ${var.description?html}
   </#if>
   <DL class="value-list">
   <#list var[".//value[not(./value)]"] as leaf>
      <DT class="value">
      <#list leaf?ancestors('value')?reverse as branch>
         ${branch.@name}-<#t>
      </#list>
         ${leaf.@name}<#t>
      </DT><#lt>
      <#if leaf.description?size != 0>
      <DD>${leaf.description?html}</DD>
      </#if>
   </#list>
   </DL>
   </DD>
</#list>
</DL>

<H2><A name="globals">Global Constraints</A></H2>

<H3><A name="ctable">Constraint Table</A></H3>

<#if doc.model[".//value_set[./@type='constrained']"]?size != 0>
   <@constraint_table/>
<#else>
   <P>(Table omitted: contains no constraints.)</P>
</#if>

<H3><A name="textglobals">Other Global Constraints</A></H3>

<#if doc.model.constraints.constraint?size != 0>
   <DL class="constraint-list">
   <#list doc.model.constraints.constraint as constraint>
      <DT class="constraint-name">${constraint.@name}</DT>
      <DD class="constraint-event">
      <@formula_text constraint.formula/>
      </DD>
   </#list>
   </DL>
</#if>

<H2><A name="events">Events: Constraints and Rules</A></H2>
<#list doc.model.events.event as event>
   <H3 class="event-name">
   <A name="event-${event.@name}">${event.@name}</A>
   </H3>
   <H4>Constraints</H4>
   <#if event.constraints.constraint?size != 0>
      <DL class="constraint-list">
      <#list event.constraints.constraint as constraint>
         <DT class="constraint-name">
         ${event.@name}&nbsp;&nbsp;&nbsp;${constraint.@name}</DT>
         <DD class="constraint-event">
         <@formula_text constraint.formula/>
         </DD>
      </#list>
      </DL>
   </#if>
   <H4>Rules</H4>
   <#if event.rules.rule?size != 0>
      <DL class="rule-list">
      <#list event.rules.rule as rule>
         <DT class="rule-name">
         ${event.@name}&nbsp;&nbsp;&nbsp;${rule.@name}
         </DT>
         <DD class="rule">
         <#if rule.values_array.value_set?size != 0>
            <TABLE summary="transition rule"><#lt>
            <@table_head/>
            <#lt>   <TBODY>
            <@rule_row rule, 'source'/>
            <#if rule.values_array["value_set[@type='target']"]?size
               != 0
            >
               <@rule_row rule, 'target'/>
            </#if>
            <#lt>   </TBODY>
            </TABLE><#lt>
         </#if>
         <#if rule.formula?size != 0>
            <@formula_text rule.formula/>
         </#if>
         </DD>
      </#list>
      </DL>
   </#if>
</#list>

</BODY>
</HTML>
</#compress>
<#-----------------  Macros and Functions ----------------->

<#-- Output the model title. -->
<#macro title>
   <#if doc.model.title?size != 0>
      ${doc.model.title?html}<#t>
   <#else>
      [Untitled Statestep Model]<#t>
   </#if>
</#macro>


<#-- Output the text of a constraint or rule formula. -->
<#macro formula_text formula>
   <PRE>${formula.text?html}</PRE>
</#macro>


<#-- Output the constraint table. -->
<#macro constraint_table>
   <TABLE summary="constraint table"><#lt>
   <@table_head/>
   <TBODY>
   <#local rows_done = []/>
   <#list usable_variables as var>
      <#list var[".//value[not(./value)]"] as leaf>
         <#local found=false/>
         <#list doc.model.simple_constraints.constraint_group as row>
            <#local output = false/>
            <#local ing=row["./value_set[@type='constraining']"][0]/>
            <#if ing.@variable != var.@name>
            <#elseif ing.@content="dont_care">
               <#local output=true/>
            <#else>
               <#local leaf_id = leaf["count(preceding::*)"]/>
               <#list val_def_nodes(ing) as branch>
                  <#local vals = branch[".//value[not(./value)]"]/>
                  <#if vals?size == 0>
                     <#local vals = [branch]/>
                  </#if>
                  <#list vals as val>
                     <#local val_id = val["count(preceding::*)"]/>
                     <#if val_id = leaf_id>
                        <#local output=true/>
                     </#if>
                  </#list>
               </#list>
            </#if>
            <#if output>
               <#local found=true/>
               <#local row_id = row["count(preceding::*)"]/>
               <#if !rows_done?seq_contains(row_id)>
                  <#lt>      <TR>
                  <#local rows_done = rows_done + [row_id]/>
                  <#list usable_variables as col_var>
                     <#local vals = row
                        ["./value_set[@variable='${col_var.@name}']"]
                     />
                     <#if vals?size != 0>
                        <#lt>         <TD class="${vals.@type}">
                        <@cell_vals vals/>
                     <#else>
                        <#lt>         <TD class="constrained">
                        &nbsp;<#t>
                     </#if>
                     </TD><#lt>
                  </#list>
                  <#lt>      </TR>
               </#if>
            </#if>
         </#list>
         <#if !found>
            <#lt>      <TR>
            <#list usable_variables as col_var>
               <#if col_var.@name = var.@name>
                  <#lt>         <TD class="constraining">
                  ${value_id(leaf)}<#t>
               <#else>
                  <#lt>         <TD class="constrained">
                  &nbsp;<#t>
               </#if>
               </TD><#lt>
            </#list>
            <#lt>      </TR>
         </#if>
      </#list>
   </#list>
   </TBODY>
   </TABLE><#lt>
</#macro>


<#-- Output a table header with the names of all the variables. -->
<#macro table_head>
   <THEAD><TR>
   <#list usable_variables as var>
      <TH>${var.@name}</TH>
   </#list>
   </TR></THEAD>
</#macro>


<#--
Output a row of the table part of a rule.
The class parameter identifies the row ('source' or 'target').
-->
<#macro rule_row rule class>
   <TR class="${class}-row">
   <#list usable_variables as var>
      <TD class="${class}">
      <#assign vals =
         rule["./values_array/value_set[@variable='${var.@name}'][@type='${class}']"]
      />
      <#if vals?size != 0>
         <@cell_vals vals/>
      <#else>
         &nbsp;
      </#if>
      </TD><#lt>
   </#list>
   </TR>
</#macro>


<#--
Output a set of variable values as a list suitable for
the contents of a table cell.
-->
<#macro cell_vals valset>
   <#switch valset.@content>
      <#case "no_change">
         =<#t>
         <#break>
      <#case "dont_care">
         <#local var = doc.model.variables
            ["variable[@name='${valset.@variable}']"][0]
         />
         <#if var.value?size = 1>
            ${value_id(most_distinguished(var.value[0]))}
         <#else>
            *<#t>
         </#if>
         <#break>
      <#case "list">
         <#list val_def_nodes(valset) as val>
            ${value_id(val)}<#t>
            <#if val_has_next>
               <BR><#lt>
            </#if>
         </#list>
         <#break>
   </#switch>
</#macro>


<#--
Given a value node in a variable definition tree,
i.e., below a 'variable' element, return the string
identifying that value or group of values.
-->
<#function value_id def_node>
   <#local id = def_node.@name/>
   <#list def_node?ancestors("value") as group>
      <#local id = group.@name + "-" + id/>
   </#list>
   <#if def_node.value?size != 0>
      <#local id = id + "-*"/>
   </#if>
   <#return id/>
</#function>


<#--
Given a set of value nodes in a value set,
return the corresponding nodes in the variable
value definition tree.
-->
<#function val_def_nodes value_set>
   <#local result = []/>
   <#local varName = value_set.@variable/>
   <#local var =
      doc.model.variables["variable[@name='${varName}']"][0]
   />
   <#list value_set[".//value[not(./value)]"] as group>
      <#local def_node = var/>
      <#local path_to_group = group?ancestors("value")?reverse
         + [group]
      />
      <#list path_to_group as component>
         <#local name=component.@name/>
         <#local def_node = def_node["./value[@name='${name}']"][0]/>
      </#list>
      <#local def_node = most_distinguished(def_node)/>
      <#local result = result + [def_node]/>
   </#list>
   <#return result/>
</#function>


<#--
Given a value node (in a variable definition tree), return
the deepest node in that tree to which it is equivalent.
Example: if only two values, a-f-p-x and a-f-p-q, are defined
for a variable and the node corresponding to branch 'a', 'f'
or 'p' is passed in then the node for 'p' will be returned.
In other words, keep descending the tree until a node with
more than one child is encountered.
-->
<#function most_distinguished val_node>
   <#if val_node.value?size = 1>
      <#local val_node = most_distinguished(val_node.value[0])/>
   </#if>
   <#return val_node/>
</#function>
